@@ -88,8 +88,9 @@ static int _memlock_count_daemon = 0;
 static int _priority;
 static int _default_priority;
 
-/* list of maps, that are unconditionaly ignored */
+/* list of maps, that are unconditionally ignored */
 static const char _ignore_maps[][16] = {
+	"[aio]",
 	"[vdso]",
 	"[vsyscall]",
 	"[vectors]",
@@ -127,6 +128,10 @@ static const char * const _blacklist_maps[] = {
 	"/libuuid.so.",		/* not using uuid during mlock (blkid) */
 	"/libz.so.",		/* not using during mlock (udev) */
 	"/libzstd.so.",		/* not using zstd during mlock (systemd) */
+	"/libjson-c.so.",	/* not using json during mlock (systemd) */
+	"/libkeyutils.so.",	/* not using keyutils during mlock (systemd) */
+	"/libcrypto.so.",	/* not using crypto during mlock (systemd) */
+	"/libnvme.so.",		/* not using nvme during mlock */
 	"/etc/selinux",		/* not using selinux during mlock */
 	/* "/libdevmapper-event.so" */
 };
@@ -179,7 +184,7 @@ static void _allocate_memory(void)
 	/* FIXME else warn user setting got ignored */
 
 #ifdef HAVE_MALLINFO2
-        /* Prefer mallinfo2 call when avaialble with newer glibc */
+        /* Prefer mallinfo2 call when available with newer glibc */
 #define MALLINFO mallinfo2
 #else
 #define MALLINFO mallinfo
@@ -353,7 +358,7 @@ static int _memlock_maps(struct cmd_context *cmd, lvmlock_t lock, size_t *mstats
 		if (!_maps_buffer || len >= _maps_len) {
 			if (_maps_buffer)
 				_maps_len *= 2;
-			if (!(line = realloc(_maps_buffer, _maps_len))) {
+			if (!_maps_len || !(line = realloc(_maps_buffer, _maps_len))) {
 				log_debug_mem("Allocation of maps buffer failed.");
 				return 0;
 			}
@@ -517,6 +522,13 @@ static void _restore_priority_if_possible(struct cmd_context *cmd)
 /* Stop memory getting swapped out */
 static void _lock_mem(struct cmd_context *cmd)
 {
+	if (!_size_stack || !_size_malloc_tmp) {
+		log_debug_mem("Skipping memory locking (reserved memory: "
+			      FMTsize_t "  stack: " FMTsize_t ").",
+			      _size_malloc_tmp, _size_stack);
+		return;
+	}
+
 	if (!cmd->running_on_valgrind)
 		 _allocate_memory();
 	(void)strerror(0);		/* Force libc.mo load */
@@ -525,7 +537,7 @@ static void _lock_mem(struct cmd_context *cmd)
 
 	/*
 	 * For daemon we need to use mlockall()
-	 * so even future adition of thread which may not even use lvm lib
+	 * so even future addition of thread which may not even use lvm lib
 	 * will not block memory locked thread
 	 * Note: assuming _memlock_count_daemon is updated before _memlock_count
 	 */
@@ -557,6 +569,13 @@ static void _unlock_mem(struct cmd_context *cmd)
 {
 	size_t unlock_mstats = 0;
 
+	if (!_size_stack || !_size_malloc_tmp) {
+		log_debug_mem("Skipping memory unlocking (reserved memory: "
+			      FMTsize_t "  stack: " FMTsize_t ").",
+			      _size_malloc_tmp, _size_stack);
+		return;
+	}
+
 	log_very_verbose("Unlocking memory");
 
 	if (!_memlock_maps(cmd, LVM_MUNLOCK, &unlock_mstats))
@@ -614,7 +633,7 @@ static void _unlock_mem_if_possible(struct cmd_context *cmd)
  * remains fast.
  *
  * Memory stays locked until 'memlock_unlock()' is called so when possible
- * it may stay locked across multiple crictical section entrances.
+ * it may stay locked across multiple critical section entrances.
  */
 void critical_section_inc(struct cmd_context *cmd, const char *reason)
 {
